#!/usr/bin/env python
# Copyright (c) 2011 Cloudera, Inc. All rights reserved.

import logging
import avro.ipc
import httplib
from M2Crypto import httpslib
from M2Crypto import SSL

LOG = logging.getLogger(__name__)


class HTTPSTransceiver(avro.ipc.HTTPTransceiver):
  '''
  A subclass of HTTPTransceiver that supports TLS
  secured traffic.

  Note that HTTPSTransceiver does not have any
  timeouts.

  It uses M2Crypto's HTTPSConnection in all cases,
  though this has not always been true.

  1) No client or server validation
  M2Crypto's HTTPSConnection will be used.

  2) Only Server validation
  In this case, pass a ca_certs_file only.
  M2Crypto's HTTPSConnection will be used.

  3) Client and Server validation
  In this case, pass all the files.
  M2Crypto's HTTPSConnection will be used.

  4) Only Client validation (kinda)
  In this case, pass all the files except the ca_certs_file.
  M2Crypto's HTTPSConnection will be used.

  Note, however, that M2Crypto does some basic sanity checking
  of the server's certificate - most notably that the CN matches
  the FQDN. An arbitrary toy certificate may not pass this test,
  so some effort is still required.

  Historical note: for a long time, we used httplib's
  HTTPSConnection for case (1), where no validation
  was required.  HTTPSConnection, in python 2.4,
  has an unfortunate interaction with global
  socket timeouts.  We already work around this
  timeout issue in m2crypto, so it was easier
  to use m2crypto in all cases.
  https://twiki.cern.ch/twiki/bin/view/CDS/PythonGotchas#3_2_Incompatibility_between_SSL
  Here's the likely py2.4 bug:
  http://bugs.python.org/issue1380952
  '''
  def __init__(self, host, port, ca_certs_file,
               key_file, cert_file, key_pw_file):

    self.key_password = None
    if key_pw_file != None:
      try:
        try:
            file = open(key_pw_file, 'r')
            self.key_password = file.readline().strip('\n')
        except Exception, e:
            LOG.exception('Failed to open key password file: ' + e)
      finally:
        file.close()

    ctx = SSL.Context('tlsv1')

    if key_file != None and cert_file != None:
      # Do client validation
      ctx.load_cert_chain(cert_file, key_file,
                          lambda * arg, **kw: self.key_password)
    if ca_certs_file == None:
      # No server validation
      ctx.set_verify(SSL.verify_none, 0)
    else:
      # Do server validation
      ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
      ctx.load_verify_info(ca_certs_file)

    self.conn = httpslib.HTTPSConnection(host, port,
                                         ssl_context=ctx)
    self.conn.connect()
